// Copyright (C) 2012 - prateek Papriwal
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function y = distfun_binopdf(varargin)
    // Binomial PDF
    //
    //Calling Sequence
    //y = distfun_binopdf(x,N,pr) 
    //
    //Parameters
    //   x : a 1x1 or nxm matrix of doubles, the number of Bernoulli trials after in which success occurs . x belongs to the set {0,1,2,3,.....}
    //   N : a 1x1 or nxm matrix of doubles , the total number of binomial trials . N belongs to the set {1,2,3,4,.......}
    //   pr : a 1x1 or nxm matrix of doubles,  the probability of success in a Bernoulli trial 
    //   y : a nxm matrix of doubles, the probability density.
    //
    // Description
    //   Computes the probability distribution function of 
    //   the Binomial distribution function.
    //   
    //   Any scalar input argument is expanded to a matrix of doubles 
    //   of the same size as the other input arguments.
    
    
    //Examples
    // // Check with x scalar, N scalar, pr scalar
    //y = distfun_binopdf(0,200,0.02);
    //expected = 0.0175879;
    
    // // Check with expanded x
    //computed = distfun_binopdf([5 15],100,0.05);
    //expected = [0.1800178 0.0000988];
    
    // // Check with expanded N
    //computed = distfun_binopdf(5,[50 100],0.05);
    //expected = [0.0658406 0.1800178];
    
    // // Check with two arguments expanded 
    //computed = distfun_binopdf([5 10],[50 100],0.05);
    //expected = [0.0658406 0.0167159];
    
    // // Check with all the arguments expanded
    //computed = distfun_binopdf([5 10],[50 100],[0.05 0.1]);
    //expected = [0.0658406 0.1318653];
    
    // // Plot the function
    // // Note that geometric distribution is discrete in nature 
    // // i.e it gives output only at discrete values of x = {0,1,2,3,.....}
    
    // scf();
    // N1 = 20;
    // x = 0:N1
    // y1 = distfun_binopdf(x,N1,0.5);
    // plot(x,y1,"bo-");
    // N2 = 20;
    // x = 0:N2
    // y2 = distfun_binopdf(x,N2,0.7);
    // plot(x,y2,"go-");
    // N3 = 40;
    // x = 0:N3;
    // y3 = distfun_binopdf(x,N3,0.5)
    // plot(x,y3,"ro-");
    
    // Bibliography
    // http://en.wikipedia.org/wiki/Binomial_distribution
    //http://forge.scilab.org/index.php/p/specfun/source/tree/HEAD/macros/specfun_nchoosek.sci
    //
    // Authors
    // Copyright (C) 2012 - prateek Papriwal
    
    [lhs,rhs] = argn()
    apifun_checkrhs("distfun_binopdf",rhs,3)
    apifun_checklhs("distfun_binopdf",lhs,0:1)
    
    x = varargin(1)
    N = varargin(2)
    pr = varargin(3)
    
    //
    // Check type
    //
    apifun_checktype("distfun_binopdf",x,"x",1,"constant")
    apifun_checktype("distfun_binopdf",N,"N",2,"constant")
    apifun_checktype("distfun_binopdf",pr,"P",3,"constant")
    //
    apifun_checkflint("distfun_binopdf",N,"N",2)
    //
    // Check content
    //  
    apifun_checkgreq("distfun_binopdf",x,"x",1,0)
    apifun_checkgreq("distfun_binopdf",N,"N",2,1)
    apifun_checkrange("distfun_binopdf",pr,"P",3,0,1)
    
    
    [x,N,pr] = apifun_expandvar(x,N,pr)
    
    r = gammaln ( N + 1 ) - gammaln (x + 1) - gammaln (N - x + 1)
    r = exp( r )
    r = round ( r )
    y = r .* (pr .^ x) .* ((1-pr) .^(N-x))
endfunction