
// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2008 - INRIA - Michael Baudin
// Copyright (C) 2009 - Digiteo - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

#include <limits.h>

#include "api_scilab.h"
#include "stack-c.h" 
#include "Scierror.h"
#include "localization.h"

#include "gwsupport.h" 
#include "gw_distfuncdf_support.h" 


// Check that P+Q=1
//
// Arguments
// fname (input): the name of the function
// ivar (input): the index of the input argument
// lrP (output): the data for P
// lrQ (output): the data for Q
// rowsP (output): the number of rows in P (and in Q)
// colsP (output): the number of columns in P (and in Q)
//
// Description
// If the matrix is not in the range, produce an error and returns 0.
// If the matrix is OK, returns 1.
//
int distfun_CheckPQEqualOne (char *fname, double* lrP, double* lrQ, int rowsP , int colsP)
{
	double p, q;
	double pq;
	int i;
	double eps;

	eps = dlamch_("p", (long int)1);

	for ( i=0 ; i < rowsP*colsP; i++)
	{
		p = *(lrP + i );
		q = *(lrQ + i );
		pq = p+q;
		if (fabs(pq-1.) > 3*eps )
		{
			Scierror(999,_("%s: P + Q not equal to 1.\n"),fname);
			return 0;
		}
	}
	return 1;
}
// Get the matrix of doubles of P.
//
// Arguments
// fname (input): the name of the function
// ivar (input): the index of the input argument
// rowsExpected (input): the number of expected rows
// colsExpected (input): the number of expected columns
// lrP (output): the data
// rowsP (output): the number of rows
// colsP (output): the number of columns
//
// Description
// If we cannot get the matrix, produce an error and returns 0.
// Checks that the matrix has the expected size.
// Checks that the matrix has entries in [0,1].
// If we can get the matrix, returns 1.
//
int distfun_GetMatrixP( char * fname, int ivar , int rowsExpected, int colsExpected ,
	double** lrP, int * rowsP , int * colsP	)
{
	int readFlag = 0;

	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, ivar, lrP, rowsP , colsP );
	if(readFlag==0)
	{
		return 0;
	}
	readFlag = gwsupport_CheckSize ( fname , ivar, rowsExpected , colsExpected , *rowsP , *colsP );
	if (readFlag==0)
	{
		return 0;
	}
	readFlag = gwsupport_CheckDoubleMatrixInRange (fname, ivar, *lrP, *rowsP , *colsP, 0., 1.);
	if (readFlag==0)
	{
		return 0;
	}
	return 1;
}
// Produce a generic error message, given the status of 
// the Inverse CDF function in DCDFLIB.
//
// Arguments
// fname (input): the name of the function
// ivar (input): the index of the input argument
// ivar (input): the index of the input argument
//
// Description
// In the case where status is different from zero, an 
// error must be generated.
// This function manages the following status values:
// status=1 : Result appears to be lower than lowest search bound
// status=2 : Result appears to be greater than greatest search bound
// status=3 : P + Q .ne. 1
// status<0 : Input argument %d of routine is out of range
//
// The status=2 error should not appear in practice:
// this is checked in the gateway, before we call the DCDFLIB function.
//
// Specific status not managed by distfun_invCDFError must be done 
// before the call to distfun_invCDFError.
//
int distfun_defaultCDFError( char * fname, int status )
{
	switch ( status )
	{
	case 1 : 
		{
			Scierror(999,_("%s: Internal error: Result appears to be lower than lowest search bound\n"), fname);
		}
		break;
	case 2 : 
		{
			Scierror(999,_("%s: Internal error: Result appears to be greater than greatest search bound\n"), fname);
		}
		break;
	case 3 : 
		{
			Scierror(999,_("%s: Internal error: P + Q not equal to 1 \n"), fname);
		}
		break;
	default :
		{
			Scierror(999, _("%s: Internal error: Input argument %d of internal function is out of range\n"), fname, -status );
		}
		break;
	}
	return 0;
}

// Get the two matrices P (argument #ivar) and Q (argument #(ivar+1)) 
//
// Arguments
// fname (input): the name of the function
// ivar (input): the index of the input argument
// rowsExpected (input): the number of expected rows
// colsExpected (input): the number of expected columns
// lrP (output): the data for P
// rowsP (output): the number of rows in P
// colsP (output): the number of columns in P
// lrQ (output): the data in Q
// rowsQ (output): the number of rows in Q
// colsQ (output): the number of columns in Q
//
// Description
// If we cannot get both matrices, produce an error and returns 0.
// Checks that the matrices P and Q have entries in [0,1].
// If rowsExpected > 0, checks that the matrices have the expected size.
// Checks that P + Q ~ 1.
// If we can get either P or Q, returns 1.
//
int distfun_GetMatricesPQ( char * fname, int ivar, int rowsExpected, int colsExpected ,
	double** lrP, int * rowsP , int * colsP, 
	double** lrQ, int * rowsQ , int * colsQ )
{
	int readFlag;

	// Arg #ivar : P
	readFlag = distfun_GetMatrixP( fname, ivar, rowsExpected, colsExpected, lrP, rowsP , colsP );
	if(readFlag==0)
	{
		return 0;
	}
	// Arg #(ivar+1) : Q
	readFlag = distfun_GetMatrixP( fname, ivar+1, rowsExpected, colsExpected, lrQ, rowsQ , colsQ );
	if(readFlag==0)
	{
		return 0;
	}
	// Check P+Q ~ 1
	readFlag = distfun_CheckPQEqualOne (fname, *lrP, *lrQ, *rowsP, *colsP);
	if (readFlag==0)
	{
		return 0;
	}
	return 1;
}

// Check that X+Y=1
//
// Arguments
// fname (input): the name of the function
// ivar (input): the index of the input argument
// lrX (output): the data for X
// lrY (output): the data for Y
// rowsX (output): the number of rows in X (and in Y)
// colsX (output): the number of columns in X (and in Y)
//
// Description
// If the matrix is not in the range, produce an error and returns 0.
// If the matrix is OK, returns 1.
//
int distfun_CheckXYEqualOne (char *fname, double* lrX, double* lrY, int rowsX , int colsX)
{
	double x, y;
	double xy;
	int i;
	double eps;

	eps = dlamch_("p", (long int)1);

	for ( i=0 ; i < rowsX*colsX; i++)
	{
		x = *(lrX + i );
		y = *(lrY + i );
		xy = x+y;
		if (fabs(xy-1.) > 3*eps )
		{
			Scierror(999,_("%s: X + Y not equal to 1.\n"),fname);
			return 0;
		}
	}
	return 1;
}

// Get a matrix of doubles >=0
//
// Arguments
// fname (input): the name of the function
// ivar (input): the index of the input argument
// rowsExpected (input): the number of expected rows
// colsExpected (input): the number of expected columns
// lrP (output): the data
// rowsP (output): the number of rows
// colsP (output): the number of columns
//
// Description
// If we cannot get the matrix, produce an error and returns 0.
// Checks that the matrix has entries greater or equal to 0.
// If rowsExpected > 0, checks that the matrix has the expected size.
// If we can get the matrix, returns 1.
//
int distfun_GetSizedMatrixOfDoublesGreaterOrEqualThanZero( char * fname, int ivar , int rowsExpected, int colsExpected ,
	double** _pdblReal, int * rowsA , int * colsA	)
{
	int readFlag;

	readFlag = gwsupport_GetMatrixOfDoublesGreaterOrEqualThanZero( fname, ivar , _pdblReal, rowsA , colsA	);
	if(readFlag==0)
	{
		return 0;
	}
	if ( rowsExpected > 0 )
	{
		readFlag = gwsupport_CheckSize ( fname , ivar, rowsExpected , colsExpected , *rowsA , *colsA );
		if (readFlag==0)
		{
			return 0;
		}
	}
	return 1;
}

// Get a matrix of doubles >0
//
// Arguments
// fname (input): the name of the function
// ivar (input): the index of the input argument
// rowsExpected (input): the number of expected rows
// colsExpected (input): the number of expected columns
// lrP (output): the data
// rowsP (output): the number of rows
// colsP (output): the number of columns
//
// Description
// If we cannot get the matrix, produce an error and returns 0.
// Checks that the matrix has entries greater or equal to 0.
// If rowsExpected > 0, checks that the matrix has the expected size.
// If we can get the matrix, returns 1.
//
int distfun_GetSizedMatrixOfDoublesGreaterThanZero( char * fname, int ivar , int rowsExpected, int colsExpected ,
	double** _pdblReal, int * rowsA , int * colsA	)
{
	int readFlag;

	readFlag = gwsupport_GetMatrixOfDoublesGreaterThanZero( fname, ivar , _pdblReal, rowsA , colsA	);
	if(readFlag==0)
	{
		return 0;
	}
	if ( rowsExpected > 0 )
	{
		readFlag = gwsupport_CheckSize ( fname , ivar, rowsExpected , colsExpected , *rowsA , *colsA );
		if (readFlag==0)
		{
			return 0;
		}
	}
	return 1;
}
