
// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2008 - INRIA - Michael Baudin
// Copyright (C) 2009 - Digiteo - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

#include <limits.h>

#include "api_scilab.h"
#include "stack-c.h" 
#include "Scierror.h"
#include "localization.h"

#include "gw_distfungrand_support.h" 
#include "gwsupport.h" 


int distfun_CheckExpansionCaseA ( char * fname , int m, int n, int ma , int na );
int distfun_CheckExpansionCaseAB ( char * fname , int m, int n, int ma , int na , int mb , int nb );
int distfun_CheckExpansionCaseABC ( char * fname , int m, int n, int ma , int na , int mb , int nb , int mc , int nc );
int distfun_WhatParameterSize ( char * fname , int m, int n, int ma , int na );


// 
// distfun_GetMN --
//   Returns the m and n scalars for distfun_grand(nrows,ncols,...) functions.
// Arguments
//   fname (input) : the name of the Scilab function generating this error
//   nrows (output) : the required number of rows
//   ncols (output) : the required number of columns
// Description
//   Returns 0 if an error is detected, returns 1 if no error occurs.
//   Reports an error if the actual number of rows 
//   is not equal to 1.
//   Reports an error if the actual number of columns 
//   is not equal to 1.
//
int distfun_GetMN ( char * fname , int * nrows , int * ncols )
{
	int readFlag;
	double doubleNrows;
	double doubleNcols;

	// Get Arg #1 = M
	readFlag = gwsupport_GetOneIntegerArgument ( fname , 1, nrows );
	if ( readFlag == 0)
	{ 
		return 0;
	}
	doubleNrows = (double) (*nrows);
	readFlag = gwsupport_CheckDoubleGreaterOrEqual (fname, 1, doubleNrows, 0.0);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	// Get Arg #2 = N
	readFlag = gwsupport_GetOneIntegerArgument ( fname , 2, ncols );
	if ( readFlag == 0)
	{ 
		return 0;
	}
	doubleNcols = (double) (*ncols);
	readFlag = gwsupport_CheckDoubleGreaterOrEqual (fname, 2, doubleNcols, 0.0);
	if ( readFlag == 0)
	{ 
		return 0;
	}
	return 1;
}

// 
// distfun_WhatParameterSize --
//   Returns flag=1 if size(a)=[1 1], 
//   returns flag=2 if size(a)=[m n], 
//   returns flag=0 if the size of the parameter a is wrong.
// Arguments
//   fname (input) : the name of the Scilab function generating this error
//   m (input) : the required number of rows
//   n (input) : the required number of columns
//   ma (input) : the number of rows in a
//   na (input) : the number of columns in a
// Description
// Use this function to expand parameters.
//
int distfun_WhatParameterSize ( char * fname , int m, int n, int ma , int na )
{
	int flag;
	if ( ma==1 && na==1 ) 
	{ 
		flag = 1;
	} 
	else if ( ma==m && na==n ) 
	{ 
		flag = 2;
	} 
	else
	{
		flag = 0;
	}
	return flag;
}

// 
// distfun_CheckExpansionCaseA --
//   Returns expcase=1 if (a is 1-by-1 matrix),
//   or expcase=2 if (size(a) == m-by-n)
//   or expcase=0 (error) if no case match.
// Arguments
//   fname (input) : the name of the Scilab function generating this error
//   m (input) : the number of rows required by the user
//   n (input) : the number of columns required by the user
//   ma (input) : the number of rows in a
//   na (input) : the number of columns in a
// Description
// Use this function to expand distfun_grandXXX(m,n,a) functions 
// to the case where a is non-scalar.
//
int distfun_CheckExpansionCaseA ( char * fname , int m, int n, int ma , int na )
{
	int expcase;
	int flag_a;
	flag_a = distfun_WhatParameterSize ( fname , m, n, ma , na );

	// Check the cases
	if ( flag_a==1 ) 
	{ 
		expcase = 1;
	} 
	else if ( flag_a == 2 )
	{
		expcase = 2;
	}
	else
	{
		// a is wrong
		Scierror(999,_("%s: Wrong size for input argument #%d: %d-by-%d matrix expected.\n"),
			fname, 3 , m, n );
		expcase = 0;
	}
	return expcase;
}

// 
// distfun_CheckExpansionCaseAB --
//   Returns expcase=1 if (a and b are 1-by-1 matrices),
//   or expcase=2 if (size(a) == m-by-n and size(b)== m-by-n)
//   or expcase=0 (error) if no case match.
// Arguments
//   fname (input) : the name of the Scilab function generating this error
//   ma (input) : the number of rows in a
//   na (input) : the number of columns in a
//   mb (input) : the number of rows in b
//   nb (input) : the number of columns in b
// Description
// Use this function to expand distfun_grandXXX(m,n,a,b) functions 
// to the case where a and b are non-scalars.
//
int distfun_CheckExpansionCaseAB ( char * fname , int m, int n, int ma , int na , int mb , int nb )
{
	int expcase;
	int flag_a;
	int flag_b;
	flag_a = distfun_WhatParameterSize ( fname, m, n, ma, na );
	flag_b = distfun_WhatParameterSize ( fname, m, n, mb, nb );

	// Check the cases
	if ( flag_a==1 && flag_b==1 )
	{ 
		expcase = 1;
	} 
	else if ( flag_a==2 && flag_b==2 ) 
	{
		expcase = 2;
	}
	else
	{
		if ( flag_a==0 )
		{
			// a is wrong
			Scierror(999,_("%s: Wrong size for input argument #%d: %d-by-%d matrix expected.\n"),
				fname, 3 , m, n );
			expcase = 0;
		}
		else
		{
			// b is wrong
			Scierror(999,_("%s: Wrong size for input argument #%d: %d-by-%d matrix expected.\n"),
				fname, 4 , m, n );
			expcase = 0;
		}
	}
	return expcase;
}
// 
// distfun_CheckExpansionCaseABC --
//   Returns expcase=1 if (a and b and c are 1-by-1 matrices),
//   or expcase=2 if (size(a) == m-by-n and size(b)== m-by-n and size(c)== m-by-n)
//   or expcase=0 (error) if no case match.
// Arguments
//   fname (input) : the name of the Scilab function generating this error
//   ma (input) : the number of rows in a
//   na (input) : the number of columns in a
//   mb (input) : the number of rows in b
//   nb (input) : the number of columns in b
//   mc (input) : the number of rows in c
//   nc (input) : the number of columns in c
// Description
// Use this function to expand distfun_grandXXX(m,n,a,b,c) functions 
// to the case where a and b are non-scalars.
//
int distfun_CheckExpansionCaseABC ( char * fname , int m, int n, int ma , int na , int mb , int nb , int mc , int nc )
{
	int expcase;
	int flag_a;
	int flag_b;
	int flag_c;
	flag_a = distfun_WhatParameterSize ( fname, m, n, ma, na );
	flag_b = distfun_WhatParameterSize ( fname, m, n, mb, nb );
	flag_c = distfun_WhatParameterSize ( fname, m, n, mc, nc );

	// Check the cases
	if ( flag_a==1 && flag_b==1 && flag_c==1 ) 
	{ 
		expcase = 1;
	} 
	else if (flag_a==2 && flag_b==2 && flag_c==2 )
	{
		expcase = 2;
	}
	else
	{
		if ( flag_a==0 )
		{
			// a is wrong
			Scierror(999,_("%s: Wrong size for input argument #%d: %d-by-%d matrix expected.\n"),
				fname, 3 , m, n );
			expcase = 0;
		}
		else if ( flag_b==0 )
		{
			// b is wrong
			Scierror(999,_("%s: Wrong size for input argument #%d: %d-by-%d matrix expected.\n"),
				fname, 4 , m, n );
			expcase = 0;
		}
		else
		{
			// c is wrong
			Scierror(999,_("%s: Wrong size for input argument #%d: %d-by-%d matrix expected.\n"),
				fname, 5 , m, n );
			expcase = 0;
		}
	}
	return expcase;
}



// Compute the result as the matrix of random numbers for 2 parameters A and B
//
// Arguments
// fname (input): the name of the function
// ivar (input): the index of the input argument
// ma (input): the number of rows in a
// na (input): the number of columns in a
// pa (input): the data in a
// mb (input): the number of rows in b
// nb (input): the number of columns in b
// pb (input): the data in b
// rRows (input): the number of rows in R
// rCols (input): the number of columns in R
// pr (input): the data in R
// randgen (input): the random number generator, with calling sequence
//   R = randgen(pa,pb)
//
// Description
// If the computation is not possible, produces an error and returns 0.
// If the computation is OK, returns 1.
//

int distfun_computeRandgenAB (char *fname, 
	int ma, int na, double * pa, 
	int mb, int nb, double * pb, 
	int rRows, int rCols,  
	double (*randgen)(double * pa , double * pb) )
{
	int i;
	int expcase;
	double * pr = NULL;
	double randvalue;

	// See the expansion case
	expcase = distfun_CheckExpansionCaseAB ( fname , rRows, rCols, ma , na , mb , nb );
	if ( expcase == 0)
	{ 
		return 0;
	}
	// Create Lhs
	gwsupport_CreateLhsMatrixOfDoubles ( 1, rRows, rCols, &pr);
	// Now compute the result
	if ( expcase == 1)
	{
		// a and b are scalars
		for ( i=0 ; i < rRows*rCols ; i++)
		{
			randvalue = (*randgen)(pa,pb);
			*(pr+i) = randvalue;
		}
	}
	else
	{
		// a and b are vectors
		for ( i=0 ; i < rRows*rCols ; i++)
		{
			randvalue = (*randgen)(pa+i,pb+i);
			*(pr+i) = randvalue;
		}
	}
	return 1;
}

// Compute the result as the matrix of random numbers for 1 parameter A
//
// Arguments
// fname (input): the name of the function
// ivar (input): the index of the input argument
// ma (input): the number of rows in a
// na (input): the number of columns in a
// pa (input): the data in a
// rRows (input): the number of rows in R
// rCols (input): the number of columns in R
// randgen (input): the random number generator, with calling sequence
//   R = randgen(pa)
//
// Description
// If the computation is not possible, produces an error and returns 0.
// If the computation is OK, returns 1.
//

int distfun_computeRandgenA (char *fname, 
	int ma, int na, double * pa, 
	int rRows, int rCols, 
	double (*randgen)(double * pa) )
{
	int i;
	int expcase;
	double * pr = NULL;
	double randvalue;

	// See the expansion case
	expcase = distfun_CheckExpansionCaseA ( fname , rRows, rCols, ma , na );
	if ( expcase == 0)
	{ 
		return 0;
	}
	// Create Lhs
	gwsupport_CreateLhsMatrixOfDoubles ( 1, rRows, rCols, &pr);
	// Now compute the result
	if ( expcase == 1)
	{
		// a is scalar
		for ( i=0 ; i < rRows*rCols ; i++)
		{
			randvalue = (*randgen)(pa);
			*(pr+i) = randvalue;
		}
	}
	else
	{
		// a is vector
		for ( i=0 ; i < rRows*rCols ; i++)
		{
			randvalue = (*randgen)(pa+i);
			*(pr+i) = randvalue;
		}
	}
	return 1;
}

// Compute the result as the matrix of random numbers for 3 parameters A and B and C
//
// Arguments
// fname (input): the name of the function
// ivar (input): the index of the input argument
// ma (input): the number of rows in a
// na (input): the number of columns in a
// pa (input): the data in a
// mb (input): the number of rows in b
// nb (input): the number of columns in b
// pb (input): the data in b
// mc (input): the number of rows in c
// nc (input): the number of columns in c
// pc (input): the data in c
// rRows (input): the number of rows in R
// rCols (input): the number of columns in R
// pr (input): the data in R
// randgen (input): the random number generator, with calling sequence
//   R = randgen(pa,pb,pc)
//
// Description
// If the computation is not possible, produces an error and returns 0.
// If the computation is OK, returns 1.
//

int distfun_computeRandgenABC (char *fname, 
	int ma, int na, double * pa, 
	int mb, int nb, double * pb, 
	int mc, int nc, double * pc, 
	int rRows, int rCols,  
	double (*randgen)(double * pa , double * pb, double * pc) )
{
	int i;
	int expcase;
	double * pr = NULL;
	double randvalue;

	// See the expansion case
	expcase = distfun_CheckExpansionCaseABC ( fname , rRows, rCols, ma , na , mb , nb , mc , nc );
	if ( expcase == 0)
	{ 
		return 0;
	}
	// Create Lhs
	gwsupport_CreateLhsMatrixOfDoubles ( 1, rRows, rCols, &pr);
	// Now compute the result
	if ( expcase == 1)
	{
		// a and b and c are scalars
		for ( i=0 ; i < rRows*rCols ; i++)
		{
			randvalue = (*randgen)(pa,pb,pc);
			*(pr+i) = randvalue;
		}
	}
	else
	{
		// a and b and c are vectors
		for ( i=0 ; i < rRows*rCols ; i++)
		{
			randvalue = (*randgen)(pa+i,pb+i,pc+i);
			*(pr+i) = randvalue;
		}
	}
	return 1;
}
