
/*
* Copyright (C)  2012 - Michael Baudin
* Copyright (C)  2001 - Bruno Pincon
* Copyright (C) ENPC
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*------------------------------------------------------------------------
*    Interface for grand
*    jpc@cermics.enpc.fr
*    stuff to deal with several generators added
*         by Bruno Pincon (12/11/2001)
*
--------------------------------------------------------------------------*/
#include <string.h>
#include <math.h>

// From Distfun:
#include "gwsupport.h"
#include "genrand.h"
#include "gw_distfungrand.h"
#include "gw_distfungrand_support.h" 
#include "unifrng.h"

// From Scilab:
#include "Scierror.h"
#include "sciprint.h"
#include "localization.h"
#include "stack-c.h"

/**************************************************
Y=distfun_grandmarkov(n,P,x0) 

generate n successive states of a Markov chain described by 
the transition matrix P. 

Initial state is given by x0 (integer values). 

If x0 is a matrix of size m=size(x0,'*') then 
Y is a matrix of size m x n. 

Y(i,:) is the sample path obtained from initial state x0(i).
***********************************************************************/

int sci_distfun_grandmarkov(char *fname,unsigned long fname_len)
{
	int minrhs = 1,maxrhs = 10,minlhs=1,maxlhs=2;
	int rRows,rCols,m2,n2,m1,n1,l1,la,lr,lb;
	int i;
	int nn,n1p1,lr1,j,icur,mm,jj;
	

	CheckRhs(3,3);
	CheckLhs(1,1);

	GetRhsVar(1,MATRIX_OF_INTEGER_DATATYPE, &rRows, &rCols, &l1);

	if ( rRows*rCols != 1)
	{ 
		Scierror(999,_("%s: Wrong value for first input argument: Must be the number of random simulation.\n"),fname);
		return 0;
	}
	nn= *istk(l1);
	GetRhsVar(2,MATRIX_OF_DOUBLE_DATATYPE, &m1, &n1, &la);
	GetRhsVar(3,MATRIX_OF_INTEGER_DATATYPE, &m2, &n2, &lb);
	if ( m1 != n1 && m1 != 1 )
	{
		Scierror(999,_("%s: Wrong second input argument: Square matrix or row vector expected.\n"), fname);
		return 0;
	}

	if ( m2*n2 == 0 ) 
	{ 
		Scierror(999,_("%s: X0 is empty\n"),fname);
		return 0;
	}

	for ( i = 0 ; i < m2*n2 ; i++)
		if ( *istk(lb+i)-1 < 0 || *istk(lb+i)-1 >= n1 )
		{
			Scierror(999,_("%s: X0(%d) must be in the range [1,%d]\n"),fname,i,n1);
			return 0;
		}
		mm= m2*n2;
		CreateVar(Rhs+1,MATRIX_OF_INTEGER_DATATYPE,&mm,&nn,&lr);

		n1p1=n1+1;
		CreateVar(Rhs+2,MATRIX_OF_DOUBLE_DATATYPE,&m1,&n1p1,&lr1);
		for ( i= 0 ; i < m1 ; i++ )
		{
			double ptot = 0.0;
			for ( j = 0 ; j < n1 ; j++ )
			{
				if ( *stk(la+i+m1*j) < 0 )
				{
					Scierror(999,_("%s: P(%d,%d) < 0\n"),fname,i+1,j+1);
					return 0;
				}
				if ( *stk(la+i+m1*j) > 1 )
				{
					Scierror(999,_("%s: P(%d,%d) > 1\n"),fname,i+1,j+1);
					return 0;
				}
				ptot += *stk(la+i+m1*j) ;
			}
			if ( fabs(ptot-1.0) > 1e-8 )
			{
				Scierror(999,_("%s: Sum of P(%d,1:%d)=%f ~= 1\n"),fname,i+1,n1,ptot);
				return 0;
			}
		}
		/** Computing the cumulative sum of the P matrix **/
		for ( i = 0 ; i < m1 ; i++)
		{
			double cumsum=0.0;
			*stk(lr1 +i) = cumsum;
			for ( j= 1; j < n1p1 ; j++ )
			{
				cumsum += *stk(la + i + m1*(j-1));
				*stk(lr1+i+m1*j) = cumsum;
			}
		}
		for ( jj = 0 ; jj < mm ; jj++)
		{
			icur = *istk(lb+jj)-1;
			for ( i=0 ; i < nn ; i++)
			{
				int niv=0;
				double rr = unifrng_rand();
				if ( m1 == 1 ) icur =0;
				while ( rr >= *stk(lr1+ icur +m1*niv) && niv < n1p1 )
				{
					niv++;
				}
				/** projection to avoid boundaries **/
				niv = Max(Min(niv,n1),1);
				*istk(lr+jj+mm*i)= niv ;
				icur=niv-1;
			}
		}
		LhsVar(1) = Rhs+1;

		return 0;
}
