
/*
* Copyright (C)  2012 - Michael Baudin
* Copyright (C)  2001 - Bruno Pincon
* Copyright (C) ENPC
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

/*------------------------------------------------------------------------
*    Interface for grand
*    jpc@cermics.enpc.fr
*    stuff to deal with several generators added
*         by Bruno Pincon (12/11/2001)
*
--------------------------------------------------------------------------*/
#include <string.h>
#include <math.h>

// From Distfun
#include "gwsupport.h"
#include "gw_distfunurng.h"
#include "unifrng.h"

// From Scilab:
#include "Scierror.h"
#include "sciprint.h"
#include "localization.h"
#include "stack-c.h"

/**************************************************
distfun_seedset(S)
distfun_seedset(s1,s2,s3,s4) 

Sets the state of the current base generator

for mt: distfun_seedset(S)
S is a vector of integers of dim 1 or 625 
The first component is an index and must be in [1,624], 
the 624 last ones must be in [0,2^32[) (but must not be all zeros) ; 
a simpler initialisation may be done with only one integer s1 (s1 must be in [0,2^32[)

for kiss: distfun_seedset(s1,s2,s3,s4) 
4 integers s1,s2, s3,s4 in [0,2^32[ must be provided ;

for clcg2: distfun_seedset(s1,s2)
2 integers s1 in [1,2147483562] and s2 in [1,2147483398] must be given ;

for clcg4: distfun_seedset(s1,s2,s3,s4) 
4 integers s1 in [1,2147483646], 
s2 in [1,2147483542], s3 in [1,2147483422], 
s4 in [1,2147483322] are required ; 
CAUTION : with clcg4 you set the seeds of the current virtual generator 
but you may lost the synchronisation between this one and the others 
virtuals generators.
In other words, the sequence generated is not warranty to be 
non over-lapping with a sequence generated by another virtual generator. 
Users should consider the distfun_urngsetall function instead.

for urand: distfun_seedset(S)
1 integer S in [0,2^31[ must be given.

for fsultra: distfun_seedset(S) or distfun_seedset(s1,s2) 
S is a vector of integers of dim 40 (the first component is an index 
and must be in [0,37], the 2d component is a flag (0 or 1), 
the 3d an integer in [1,2^32[ and the 37 others integers in [0,2^32[).
A simpler (and recommended) initialisation may be done with 
two integers s1 and s2 in [0,2^32[.
***********************************************************************/

int sci_distfun_seedset(char *fname,unsigned long fname_len)
{
	int l2,m1,n1,l1;
	int l3,l4;
	int current_gen;

	Nbvars = 0;
	CheckRhs(1,4);
	CheckLhs(0,1);

	unifrng_getcurrentgen(&current_gen);
	switch(current_gen)
	{
	case(MT) :
		CheckRhs(1,1);
		GetRhsVar(1,MATRIX_OF_DOUBLE_DATATYPE,&m1,&n1,&l1);
		if ( m1*n1 == 1)          /* simple init of mt     */
		{ 
			if (! unifrng_mt_set_state_simple(*stk(l1)) ) 
			{
				Scierror(999,_("%s: Internal error: Cannot set (single) state of MT generator.\n"),fname);
				return(0);
			} 
		}
		else if ( m1*n1 == 625 )  /* init of all the state */
		{ 
			if (! unifrng_mt_set_state(stk(l1))) 
			{
				Scierror(999,_("%s: Internal error: Cannot set (full) state of MT generator.\n"),fname);
				return(0);
			}
		}
		else
		{
			Scierror(999,_("%s: Wrong values for input argument: Vector of %d or %d values for %s expected.\n"),fname,1, 625,"mt");
			return 0;
		}
		break;

	case(FSULTRA) :
		CheckRhs(1,2);
		if ( Rhs == 1 ) /* init via a "complete" state */
		{
			GetRhsVar(1,MATRIX_OF_DOUBLE_DATATYPE,&m1,&n1,&l1);
			if ( m1 != 40  ||  n1 != 1)
			{
				Scierror(999,_("%s: Wrong size for second input argument: %dx%d expected.\n"),fname,40,1);
				return 0;
			}
			if (! unifrng_fsultra_set_state(stk(l1)) ) 
			{
				Scierror(999,_("%s: Internal error: Cannot set state of FSULTRA generator.\n"),fname);
				return(0);
			}
		}
		else if ( Rhs == 2 ) /* init with 2 integers (like before) */
		{
			GetRhsVar(1,MATRIX_OF_DOUBLE_DATATYPE,&m1,&n1,&l1);
			if ( m1*n1 != 1)
			{ 
				Scierror(999,_("%s: Wrong type for second input argument: Scalar expected.\n"),fname); 
				return 0;
			}
			GetRhsVar(2,MATRIX_OF_DOUBLE_DATATYPE,&m1,&n1,&l2);
			if ( m1*n1 != 1)
			{ 
				Scierror(999,_("%s: Wrong type for third input argument: Scalar expected.\n"),fname); 
				return 0;
			}
			if (! unifrng_fsultra_set_state_simple(*stk(l1),*stk(l2)) ) 
			{
				Scierror(999,_("%s: Internal error: Cannot set (simple) state of FSULTRA generator.\n"),fname);
				return(0);
			}
		}
		else
		{
			Scierror(999,_("%s: Wrong number of input arguments: %d or %d expected for '%s' option with the %s generator.\n"),fname,2,3,"setsd","fsultra");
			return 0;
		}
		break;

	case(KISS) :
	case(CLCG4) :
		CheckRhs(4,4);
		GetRhsVar(1,MATRIX_OF_DOUBLE_DATATYPE,&m1,&n1,&l1);
		if ( m1*n1 != 1)
		{ 
			Scierror(999,_("%s: Wrong type for second input argument: Scalar expected.\n"),fname); 
			return 0;
		}
		GetRhsVar(2,MATRIX_OF_DOUBLE_DATATYPE,&m1,&n1,&l2);
		if ( m1*n1 != 1)
		{ 
			Scierror(999,_("%s: Wrong type for third input argument: Scalar expected.\n"),fname); 
			return 0;
		}
		GetRhsVar(3,MATRIX_OF_DOUBLE_DATATYPE,&m1,&n1,&l3);
		if ( m1*n1 != 1)
		{ 
			Scierror(999,_("%s: Wrong type for fourth input argument: Scalar expected.\n"),fname); 
			return 0;
		}
		GetRhsVar(4,MATRIX_OF_DOUBLE_DATATYPE,&m1,&n1,&l4);
		if ( m1*n1 != 1)
		{ 
			Scierror(999,_("%s: Wrong type for fifth input argument: Scalar expected.\n"),fname); 
			return 0;
		}
		if (current_gen == KISS)
		{
			if (! unifrng_kiss_set_state(*stk(l1),*stk(l2),*stk(l3),*stk(l4))) 
			{
				Scierror(999,_("%s: Internal error: Cannot set state of KISS generator.\n"),fname);
				return 0;
			}
		}
		else
		{
			if (! unifrng_clcg4_set_initial_seed(*stk(l1),*stk(l2),*stk(l3),*stk(l4)))
			{
				Scierror(999,_("%s: Internal error: Cannot set state of CLCG4 generator.\n"),fname);
				return 0;
			}
		}
		break;

	case(CLCG2) :
		CheckRhs(2,2);
		GetRhsVar(1,MATRIX_OF_DOUBLE_DATATYPE,&m1,&n1,&l1);
		if ( m1*n1 != 1)
		{ 
			Scierror(999,_("%s: Wrong type for second input argument: Scalar expected.\n"),fname); 
			return 0;
		}
		GetRhsVar(2,MATRIX_OF_DOUBLE_DATATYPE,&m1,&n1,&l2);
		if ( m1*n1 != 1)
		{ 
			Scierror(999,_("%s: Wrong type for third input argument: Scalar expected.\n"),fname); 
			return 0;
		}
		if (! unifrng_clcg2_set_state(*stk(l1),*stk(l2)))
		{ 
			Scierror(999,_("%s: Internal error: Cannot set state of CLCG2 generator.\n"),fname);
			return 0;
		}
		break;

	case(URAND) :
		CheckRhs(1,1);
		GetRhsVar(1,MATRIX_OF_DOUBLE_DATATYPE,&m1,&n1,&l1);
		if ( m1*n1 != 1)
		{ 
			Scierror(999,_("%s: Wrong type for second input argument: Scalar expected.\n"),fname); 
			return 0;
		}
		if (! unifrng_urand_set_state(*stk(l1)))
		{
			Scierror(999,_("%s: Internal error: Cannot set state of URAND generator.\n"),fname);
			return 0;
		}
		break;
	}
	LhsVar(1) = 0;

	return 0;
}
